<?php

/**
 * @copyright Copyright © 2013-2025 Codemarket - Todos os direitos reservados
 * @author    Felipo Antonoff <contato@codemarket.com.br>
 * @link      https://www.codemarket.com.br
 * @link      https://felipoantonoff.com
 * @date      01/04/2025 17:54
 **/

namespace Codemarket\System\Library\CodeApp;

/**
 * © Copyright 2013-2025 Codemarket - Todos os direitos reservados.
 *
 * @package CodeApp
 */
class CustomField
{
    private $log;
    private $registry;
    private $db;

    /**
     * Construtor que chama o setup() para criar as tabelas e setar as variáveis.
     *
     * @param $registry
     * @param $db
     */
    public function __construct($registry)
    {
        /*
        $this->load->model('module/codemarket_module');
        $conf = $this->model_module_codemarket_module->getModulo('');
        */

        $this->log = new \log('CodeApp-Custom-Field-' . date('d-m-Y') . '.log');
        $this->setup($registry);
    }

    /**
     * setup - Cria a tabela code_custom_field.
     *
     * @return void
     */
    private function setup($registry)
    {
        $this->registry = $registry;
        $this->db = $registry->get('db');

        // CRIANDO A TABELA AUXILIAR DE CAMPOS EXTRAS
        $sql = "CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "code_custom_field` (
            `code_custom_field_id` int(11) unsigned NOT NULL AUTO_INCREMENT COMMENT 'ID do Campo Extra',
            `table_ref_name` varchar(80) DEFAULT NULL COMMENT 'Nome da Tabela relacionada ao Campo Extra',
            `table_ref_id` int(11) unsigned DEFAULT NULL COMMENT 'ID da Tabela relacionada ao Campo Extra',
            `name` varchar(150) DEFAULT NULL COMMENT 'Nome/Identificar do Campo Extra',
            `value` varchar(250) DEFAULT NULL COMMENT 'Valor simples do Campo Extra',
            `data` mediumtext DEFAULT NULL COMMENT 'Valor Json do Campo Extra',
            `date_added` datetime DEFAULT NULL COMMENT 'Quando foi adicionado',
            `date_modified` datetime DEFAULT NULL COMMENT 'Quando foi editado',
            PRIMARY KEY (`code_custom_field_id`) USING BTREE,
            UNIQUE KEY `unique1` (`table_ref_name`,`table_ref_id`,`name`) USING BTREE,
            KEY `index1` (`table_ref_name`,`name`,`value`) USING BTREE,
            KEY `table_ref_name` (`table_ref_name`) USING BTREE,
            KEY `table_ref_id` (`table_ref_id`) USING BTREE,
            KEY `name` (`name`) USING BTREE,
            KEY `value` (`value`) USING BTREE
          ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
        ";

        $this->db->query($sql);
        //cho $sql;

        /*
        // INDEX COMPOSTO
        $this->db->query(
            "CREATE UNIQUE INDEX code_custom_field_1
            ON `" . DB_PREFIX . "code_custom_field` (table_ref_name, table_ref_id, name);
            "
        );

        $this->db->query(
            "CREATE INDEX code_custom_field_2
            ON `" . DB_PREFIX . "code_custom_field` (table_ref_name, name, value);
            "
        );
        */
    }

    /**
     * Apenas um teste, retornando o primeiro campo extra encontrado.
     *
     * @return void
     */
    public function test()
    {
        $query = $this->db->query("SELECT * FROM `'.DB_PREFIX.'code_custom_field` LIMIT 1");
        print_r($query);

        exit("Teste Custom OK");
    }

    /**
     * @param string $table_ref_name
     * @param int    $table_ref_id
     * @param string $name
     * @param string $value
     * @param array  $data
     * @param bool   $update
     *
     * @return int
     */
    public function add(
        string $table_ref_name,
        int $table_ref_id,
        string $name,
        string $value,
        array $data,
        bool $update = true
    ): int {
        // Atualizado os Dados se existir
        if ($update) {
            $update = $this->update($table_ref_name, $table_ref_id, $name, $value, $data);
        }

        if (!empty($update)) {
            $this->log->write('add() - Atualizado Campo Extra ID: ' . $update);

            return $update;
        }

        // Adicionado o Campo Extra
        $sql = "
            INSERT INTO `" . DB_PREFIX . "code_custom_field`
            SET
            `table_ref_name` = '" . $this->db->escape($table_ref_name) . "',
            `table_ref_id` = '" . (int) $table_ref_id . "',
            `name` = '" . $this->db->escape($name) . "',
            `value` = '" . $this->db->escape($value) . "',
            `data` = '" . json_encode($data) . "',
            `date_added` = NOW(),
            `date_modified` = NOW()
        ";

        //echo $sql;
        $this->db->query($sql);
        $id = $this->db->getLastId();
        $this->log->write('add() - Adicionado o Campo Extra ID: ' . $id);

        return $id;
    }

    /**
     * Atualizado os dados.
     *
     * @param string $table_ref_name
     * @param int    $table_ref_id
     * @param string $name
     * @param string $value
     * @param array  $data
     *
     * @return int
     */
    public function update(string $table_ref_name, int $table_ref_id, string $name, string $value, array $data): int
    {
        $sql = "SELECT code_custom_field_id FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
        `table_ref_name` = '" . $this->db->escape($table_ref_name) . "' AND
        `table_ref_id` = '" . (int) $table_ref_id . "' AND
        `name` = '" . $this->db->escape($name) . "'
        LIMIT 1
        ";

        // Verificando se existe o Campo Extra
        $query = $this->db->query($sql)->row;

        if (empty($query['code_custom_field_id'])) {
            $this->log->write('update() - Nenhum campo extra encontrado');

            return 0;
        }

        // Mudando os dados
        $this->db->query(
            "
            UPDATE `" . DB_PREFIX . "code_custom_field`
            SET
            `value` = '" . $this->db->escape($value) . "',
            `data` = '" . json_encode($data) . "',
            date_modified = NOW()
            WHERE
            code_custom_field_id = '" . (int) $query['code_custom_field_id'] . "'
            "
        );

        return $query['code_custom_field_id'];
    }

    /**
     * Deletando um campo extra com base no seu ID.
     *
     * @param int $code_custom_field_id
     *
     * @return void
     */
    public function delete(int $code_custom_field_id)
    {
        $this->db->query(
            "DELETE FROM `" . DB_PREFIX . "code_custom_field` WHERE code_custom_field_id = '" . (int) $code_custom_field_id . "'"
        );
    }

    /**
     * Buscar pelo ID do Campo Extra.
     *
     * @param int $code_custom_field_id ID do Campo Extra
     *
     * @return array
     */
    public function get(int $code_custom_field_id): array
    {
        $sql = "SELECT * FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
        code_custom_field_id = '" . (int) $code_custom_field_id . "'
         LIMIT 1
        ";

        $query = $this->db->query($sql)->row;

        if (empty($query['code_custom_field_id'])) {
            $this->log->write('get() - Não encontrado campo extra ID: ' . $query['code_custom_field_id']);

            return [];
        }

        $query['data'] = json_decode($query['data'], true);

        $this->log->write('get() - Retornando a busca, campo extra ID: ' . $query['code_custom_field_id']);

        return $query;
    }

    /**
     * Busca pelo Name e pelo Name da Tabela de referência.
     *
     * @param string $table_ref_name
     * @param string $name
     * @param int    $limit
     *
     * @return array
     */
    public function getByName(string $table_ref_name, string $name, int $limit = 1): array
    {
        $sql = "SELECT * FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
            `table_ref_name` = '" . $this->db->escape($table_ref_name) . "' AND
            `name` LIKE '%" . $this->db->escape($name) . "%'
             ORDER BY code_custom_field_id DESC
             LIMIT " . $limit . "
        ";

        return $this->db->query($sql)->rows;
        //return $this->getReturn('getByName', $sql, $table_ref_name, 0);
    }

    /**
     * Busca pela Tabela de referências com base no nome da tabela, ID e nome do campo.
     *
     * @param string $table_ref_name
     * @param int    $table_ref_id
     * @param string $name
     * @param bool   $like
     *
     * @return array
     */
    public function getByTable(string $table_ref_name, int $table_ref_id, string $name, bool $like = true): array
    {
        $sql = "SELECT * FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
        `table_ref_name` = '" . $this->db->escape($table_ref_name) . "' AND
        `table_ref_id` = '" . (int) $table_ref_id . "' AND
        ";

        if ($like) {
            $sql .= "`name` LIKE '%" . $this->db->escape($name) . "%'";
        } else {
            $sql .= "`name` = '" . $this->db->escape($name) . "'";
        }

        $sql .= "ORDER BY code_custom_field_id DESC
            LIMIT 1
        ";

        return $this->getReturn('getByTable', $sql, $table_ref_name, $table_ref_id);
    }

    /**
     * Busca pela Tabela de referências sem o ID da referência, apenas com o nome da tabela, nome do campo e data.
     *
     * @param string $table_ref_name
     * @param string $name
     * @param string $value
     *
     * @return array
     */
    public function getByValue(string $table_ref_name, string $name, string $value): array
    {
        $sql = "SELECT * FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
        `table_ref_name` = '" . $this->db->escape($table_ref_name) . "' AND
        `name` LIKE '%" . $this->db->escape($name) . "%' AND
        `value` LIKE '%" . $this->db->escape($value) . "%'
         ORDER BY code_custom_field_id DESC
         LIMIT 1
        ";

        return $this->getReturn('getByValue', $sql, $table_ref_name, 0);
    }

    /**
     * Busca pela Tabela de referências sem o ID da referência, apenas com o nome da tabela, nome do campo e valor.
     *
     * @param string $table_ref_name
     * @param string $name
     * @param string $data
     *
     * @return array
     */
    public function getByData(string $table_ref_name, string $name, string $data): array
    {
        $sql = "SELECT * FROM `" . DB_PREFIX . "code_custom_field`
        WHERE
        `table_ref_name` = '" . $this->db->escape($table_ref_name) . "' AND
        `name` LIKE '%" . $this->db->escape($name) . "%' AND
        `data` LIKE '%" . $this->db->escape($data) . "%'
         ORDER BY code_custom_field_id DESC
         LIMIT 1
        ";

        return $this->getReturn('getByData', $sql, $table_ref_name, 0);
    }

    /**
     * @param string $function
     * @param string $sql
     * @param string $table_ref_name
     * @param int    $table_ref_id
     *
     * @return array
     */
    private function getReturn(string $function, string $sql, string $table_ref_name, int $table_ref_id = 0): array
    {
        $query = $this->db->query($sql)->row;

        if (empty($query['code_custom_field_id'])) {
            $this->log->write(
                $function . '() - Não encontrado campo extra para tabela e ID: ' . $table_ref_name . ' - ' . $table_ref_id
            );

            return [];
        }

        $query['data'] = codeHelperJsonDecode($query['data'], true);

        $this->log->write($function . '() - Retornando a busca, campo extra ID: ' . $query['code_custom_field_id']);
        //print_r($query); exit();

        return $query;
    }
}
